#include <String.h>
#include <Wire.h>

#define HMC5883L_ADDR 0x1e // real 7-bit i2c address

void init_sensor() {
  delay(100);
  Wire.beginTransmission(HMC5883L_ADDR);
  Wire.send(0x02); // mode register
  Wire.send(0x00); // continuous conversion
  Wire.endTransmission();
}

void read_sensor() {
  int16_t x, z, y;
  int16_t x_msb, x_lsb, z_msb, z_lsb, y_msb, y_lsb;
  int stat, ida, idb, idc;

  Wire.beginTransmission(HMC5883L_ADDR);
  Wire.send(0x03); // start multiple read
  Wire.endTransmission();

  Wire.requestFrom(HMC5883L_ADDR, 6);
  if (Wire.available() == 6) {
    x_msb = Wire.receive();
    x_lsb = Wire.receive();
    z_msb = Wire.receive();
    z_lsb = Wire.receive();
    y_msb = Wire.receive();
    y_lsb = Wire.receive();
    // ptr will wrap back to 3 at this point
  }

  Wire.beginTransmission(HMC5883L_ADDR);
  Wire.send(0x09); // start multiple read
  Wire.endTransmission();
  Wire.requestFrom(HMC5883L_ADDR, 4);
  if (Wire.available() == 4) {
    stat = Wire.receive();
    ida = Wire.receive();
    idb = Wire.receive();
    idc = Wire.receive();
  }
  
  x = x_msb << 8 | x_lsb;
  z = z_msb << 8 | z_lsb;
  y = y_msb << 8 | y_lsb;
  String s = "x=" + String(x) + " y=" + String(y) + " z=" + String(z) + " st=" + String(stat,HEX) + " id=" + char(ida) + char(idb) + char(idc);
  Serial.println(s);
}

void setup() {
  Serial.begin(115200);
  Wire.begin();
  init_sensor();
}

void loop() {
  delay(250);
  read_sensor();
}


